<?php

require_once(__DIR__ . '/../Models/HijriDate.php');

class TimetablePrinter
{
    protected $prayerLocal = array(
        "fajr" => "Fajr",
        "sunrise" => "Sunrise",
        "zuhr" => "Zuhr",
        "asr" => "Asr",
        "maghrib" => "Maghrib",
        "isha" => "Isha"
    );

    protected $headersLocal = array(
        "prayer" => "Prayer",
        "begins" => "Begins",
        "iqamah" => "Iqamah",
        "standard" => "Standard",
        "hanafi" => "Hanafi",
        "fast_begins" => "Fast Begins",
        "fast_ends" => "Fast Ends",
        "khutbah" => "Khutbah"
    );

    protected $numbersLocal = array(
        0 => '0',
        1 => '1',
        2 => '2',
        3 => '3',
        4 => '4',
        5 => '5',
        6 => '6',
        7 => '7',
        8 => '8',
        9 => '9'
    );

    protected $monthsLocal = array(
        'january' => 'January',
        'february' => 'February',
        'march' => 'March',
        'april' => 'April',
        'may' => 'May',
        'june' => 'June',
        'july' => 'July',
        'august' => 'August',
        'september' => 'September',
        'october' => 'October',
        'november' => 'November',
        'december' => 'December'
    );


    /** @var array  */
    protected $timesLocal = array(
        'date' => 'Date',
        'day' => 'Day',
        'minute' => 'Minutes',
        'hours' => 'Hours',
        'tomorrow' => 'IQAMAH UPDATE FOR TOMORROW'
    );

    /** @var string */
    protected $tableClass;

    /** @var array  */
    protected $localPrayerNames;

    /** @var array  */
    protected $localHeaders;

    /** @var array  */
    protected $localNumbers;

    /** @var  array */
    protected $localTimes;

    /** @var  HijriDate */
    protected $hijriDate;

    /** @var string  */
    protected $hijridateString;

    /** $var bool */
    protected $isVertical = false;

    /** $var bool */
    protected $isHorizontal = false;

    /**
     * TimetablePrinter constructor.
     */
    public function __construct()
    {
        $this->localPrayerNames = $this->getLocalPrayerNames();
        $this->localHeaders = $this->getLocalHeaders();
        $this->localNumbers = $this->getLocalNumbers();
        $this->localTimes = $this->getLocalTimes();

        $this->hijriDate = new HijriDate();
        $this->hijridateString = $this->hijriDate->getDate(date("d"), date("m"), date("Y"), true);
    }

    public function getTableClass()
    {
        return get_option('hideTableBorder');
    }

    public function getLocalPrayerNames()
    {
        $prayers_local = get_option('prayersLocal');

        if ( empty($prayers_local)) {
            return $this->prayerLocal;
        } elseif (count($prayers_local) != count($this->prayerLocal)) {
            delete_option( 'prayersLocal' );
            return $this->prayerLocal;
        }

        return $prayers_local;
    }

    public function getLocalHeaders()
    {
        $headers_local = get_option('headersLocal');

        if ( empty($headers_local)) {
            return $this->headersLocal;
        } elseif (count($headers_local) != count($this->headersLocal)) {
            delete_option( 'headersLocal' );
            return $this->headersLocal;
        }

        return $headers_local;
    }

    public function getLocalMonths()
    {
        $monthsLocal = get_option('monthsLocal');

        if ( empty( $monthsLocal )) {
            $monthsLocal = $this->monthsLocal;
        }

        if ( get_option("ramadan-chbox") ) {
            $monthsLocal['ramadan'] = 'Ramadan';
        } else {
            unset( $monthsLocal['ramadan'] );
        }

        return $monthsLocal;
    }

    public function getLocalNumbers()
    {
        $numbers_local = get_option('numbersLocal');

        return empty($numbers_local) ? $this->numbersLocal : $numbers_local;
    }


    public function getLocalTimes()
    {
        $times = get_option('timesLocal');

        if ( empty($times)) {
            return $this->timesLocal;
        } elseif (count($times) != count($this->timesLocal)) {
            delete_option( 'timesLocal' );
            return $this->timesLocal;
        }

        return $times;
    }

    public function setVertical($value=true)
    {
        $this->isVertical = $value;
    }

    public function setHorizontal($value=true)
    {
        $this->isHorizontal = $value;
    }

    /**
     * @param  string $mysqlDate
     * @param  string $format
     * @return string
     */
    public function formatDate($mysqlDate, $format=null)
    {
        $phpDate = strtotime($mysqlDate);
        $date =  date( get_option('date_format'), $phpDate );
        if ($format) {
            $date = date($format, $phpDate);
        }

        return $date;
    }

    /**
     * @param  string $mysqlDate
     * @return string
     */
    public function formatDateForPrayer($mysqlDate)
    {
        $wpDate = $this->formatDate($mysqlDate, get_option('time_format'));
        $result = str_split($wpDate);
        $intlDate = '';
        $this->localNumbers = $this->getLocalNumbers();
        foreach ($result as $number) {
            $intlDate .= $this->localNumbers[$number];
            if (empty($this->localNumbers[$number]) && $number !== '0') {
                $intlDate .= $number;
            }
        }

        return $intlDate;
    }

    /**
     * @param  string $month
     * @param  string $day
     * @return string
     */
    protected function getClass($month, $day)
    {
        if ($day == user_current_time('j') && $month == user_current_time('m')){
            return "class = highlight";
        }
    }

    /**
     * @param  bool $isRamadan
     * @param  array $data
     * @param  bool $azanOnly
     * @return string|null
     */
    protected function getFastingTdWithData($isRamadan, $data, $azanOnly=null)
    {

        if ($isRamadan) {
            return "<td class='fasting'>" . $this->formatDateForPrayer($data). "</td>";
        } elseif ($azanOnly) {
            return "<td>" . $this->formatDateForPrayer($data). "</td>";
        }
        return "";
    }

	/**
     * @param array $row
     *
     * @return string
     */
    protected function getNextIqamahTime(array $row, $countStart=60)
    {
        $diff = $this->getNextIqamahTimeDiff($row);

        if ($diff && $diff < $countStart) {
            return "
                <p class='nextIqamah green ".$this->tableClass."'>
                " .  $this->nextIqamah . "
                    <span class='timeLeftCountDown timeLeft ". $this->getIqamahClass( $diff )."'>" . $this->getLocalizedNumber( $diff ) . ":00 </span> ". $this->localTimes['minute'] ."
                </p>";
        }
    }

    protected function getNextIqamahTimeDiff(array $row)
    {
        $jamahTime = $this->getJamahTime( $row );
        $now = current_time( 'H:i');
        foreach ($jamahTime as $key=>$jamah) {
            $this->nextIqamah = $this->localPrayerNames[lcfirst($key)] . ' ' . $this->localHeaders['iqamah'] . ':';
            if ($jamah >$now ) {
                if ($key == 'Sunrise') {
                    $this->nextIqamah = $this->localPrayerNames[lcfirst($key)] . ':';
                }
                $toTime = strtotime( $jamah );
                $fromTime = strtotime( $now );
                $diff = round(abs($toTime - $fromTime)/60,2);

                return $diff;
            }
        }
    }

	/**
     * @param $row
     *
     * @return string
     */
    protected function getNextPrayer($row)
    {
        $now = current_time( 'H:i');

        $jamahTime = $this->getJamahTime( $row );
        foreach ($jamahTime as $jamah) {
            if ($jamah > $now ) {
                $prayer = array_search( $jamah, $row ); // asr_jamah
                $prayer = explode( '_', $prayer);
                return $prayer[0]; // asr
            }
        }
    }

	/**
     * @param array $row
     *
     * @return array
     */
    protected function getJamahTime(array $row)
    {
        $value = array( $row["fajr_jamah"], $row['sunrise'], $row["zuhr_jamah"], $row["asr_jamah"], $row["maghrib_jamah"], $row["isha_jamah"]);

        return array_combine( $this->prayerLocal, $value );

    }

    /**
     * @param array $row
     *
     * @return array
     */
    protected function getAzanTime(array $row)
    {
        $value = array( $row["fajr_begins"], $row['sunrise'], $row["zuhr_begins"], $row["asr_begins"], $row["maghrib_begins"], $row["isha_begins"]);

        return array_combine( $this->prayerLocal, $value );
    }

    /**
     * @param $numbers
     *
     * @return string
     */
    public function getLocalizedNumber($numbers)
    {
        $numbers = str_split( $numbers );
        $localNumber = "";
        foreach ($numbers as $number) {
            $localNumber .= $this->localNumbers[$number];
        }

        return $localNumber;
    }

    /**
     * @param $number
     *
     * @return string
     */
    protected function getIqamahClass($number)
    {
        if ( $number > 30 ) {
            return 'green';
        } elseif ( $number > 15 ) {
            return 'orange';
        }

        return 'red';
    }

    protected function getJamahChange(array $row)
    {
        $style = null;
        if ($this->isVertical) {
            $style = "style='display: block;'";
        }
        $timeRelated = $this->getLocalTimes();
        $print = "<div class='jamahChanges'>" . $timeRelated['tomorrow'] . "</br>";
        $prayerName = $this->getLocalPrayerNames();
        foreach($row['jamah_changes'] as $key=>$time) {
            if (! empty($key) ){
                $prayer = explode('_', $key)[0];
                $print .= "<span " . $style . ">" . $this->sanitizedPrayerName($prayer) . ": " .  $this->formatDateForPrayer($time) . "</span>";
            }
        }
        $print .= "</div>";

        return $print;
    }

    private function sanitizedPrayerName($prayer)
    {
        $prayerName = $this->getLocalPrayerNames();
        $localHeaders = $this->getLocalHeaders();

        if ($this->tomorrowIsFriday() && $prayer == 'zuhr') {
            return stripslashes($this->localHeaders['khutbah']);
        }

        return $prayerName[$prayer];
    }

    private function tomorrowIsFriday()
    {
        if ( date('D') == 'Thu' ) {
            return true;
        }
        return false;
    }

        /**
     * @param $day
     * @param $month
     * @param $year
     * @param array $dbRow
     *
     * @return string
     */
    public function getHijriDate($day, $month, $year, $dbRow)
    {
        $hijriCheckbox = get_option('hijri-chbox');
        if ( ! empty($hijriCheckbox) ) {
            if( ! empty($dbRow['hijri_date']) ) {
                $hijriDate = $dbRow['hijri_date'];
            } else {
                $hijridateArray = $this->hijriDate->getDate($day, $month, $year);
                $hijriDate = $hijridateArray['day']. ' '. $hijridateArray['month'] . ' ' . $hijridateArray['year'];
            }

            return '<span class="hijriDate">('. $hijriDate .')</span>';
        }
        return;
    }
}